@@ -1,6 +1,7 @@
 package operator2
 
 import (
+	"bytes"
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
@@ -10,6 +11,9 @@ import (
 	"strconv"
 	"strings"
 
+	"monis.app/go/openshift/controller"
+	"monis.app/go/openshift/operator"
+
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -29,8 +33,6 @@ import (
 	oauthclient "github.com/openshift/client-go/oauth/clientset/versioned/typed/oauth/v1"
 	routeclient "github.com/openshift/client-go/route/clientset/versioned/typed/route/v1"
 	routeinformer "github.com/openshift/client-go/route/informers/externalversions/route/v1"
-	"github.com/openshift/cluster-authentication-operator/pkg/boilerplate/controller"
-	"github.com/openshift/cluster-authentication-operator/pkg/boilerplate/operator"
 	"github.com/openshift/library-go/pkg/operator/events"
 	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
 	"github.com/openshift/library-go/pkg/operator/resource/resourcemerge"
@@ -39,114 +41,25 @@ import (
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
 )
 
-var deploymentVersionHashKey = operatorv1.GroupName + "/rvs-hash"
-
 const (
-	clusterOperatorName     = "authentication"
-	targetName              = "oauth-openshift" // this value must be "namespaced" to avoid using a route host that a customer may want
-	targetNamespace         = "openshift-authentication"
-	targetNameOperator      = "authentication-operator"
-	targetNamespaceOperator = "openshift-authentication-operator"
-	globalConfigName        = "cluster"
-
-	operatorSelfName       = "operator"
-	operatorVersionEnvName = "OPERATOR_IMAGE_VERSION"
-	operandVersionEnvName  = "OPERAND_IMAGE_VERSION"
-	operandImageEnvName    = "IMAGE"
-	kasServicePortEnvName  = "KUBERNETES_SERVICE_PORT_HTTPS"
-
-	machineConfigNamespace = "openshift-config-managed"
-	userConfigNamespace    = "openshift-config"
-
-	kasServiceAndEndpointName = "kubernetes"
-	kasServiceFullName        = kasServiceAndEndpointName + "." + corev1.NamespaceDefault + ".svc"
-
-	rootCAFile = "/var/run/secrets/kubernetes.io/serviceaccount/ca.crt"
-
-	systemConfigPath           = "/var/config/system"
-	systemConfigPathConfigMaps = systemConfigPath + "/configmaps"
-	systemConfigPathSecrets    = systemConfigPath + "/secrets"
-
-	// if one day we ever need to come up with something else, we can still find the old secrets and config maps
-	versionPrefix = "v4-0-"
-
-	configVersionPrefix = versionPrefix + "config-"
-
-	// secrets and config maps that we manually managed have this prefix
-	systemConfigPrefix = configVersionPrefix + "system-"
-
-	// secrets and config maps synced from openshift-config into our namespace have this prefix
-	userConfigPrefix = configVersionPrefix + "user-"
-	// idps that are synced have this prefix
-	userConfigPrefixIDP = userConfigPrefix + "idp-"
-	// templates that are synced have this prefix
-	userConfigPrefixTemplate = userConfigPrefix + "template-"
-
-	// secrets and config maps synced from openshift-config into our namespace have this path prefix
-	userConfigPath = "/var/config/user"
-	// root path for IDP data
-	userConfigPathPrefixIDP = userConfigPath + "/" + "idp"
-	// root path for template data
-	userConfigPathPrefixTemplate = userConfigPath + "/" + "template"
-
-	sessionNameAndKey = systemConfigPrefix + "session"
-	sessionMount      = systemConfigPathSecrets + "/" + sessionNameAndKey
-	sessionPath       = sessionMount + "/" + sessionNameAndKey
-
-	serviceCABase  = "service-ca"
-	serviceCAName  = systemConfigPrefix + serviceCABase
-	serviceCAKey   = serviceCABase + ".crt"
-	serviceCAMount = systemConfigPathConfigMaps + "/" + serviceCAName
-	serviceCAPath  = serviceCAMount + "/" + serviceCAKey
-
-	servingCertName     = systemConfigPrefix + "serving-cert"
-	servingCertMount    = systemConfigPathSecrets + "/" + servingCertName
-	servingCertPathCert = servingCertMount + "/" + corev1.TLSCertKey
-	servingCertPathKey  = servingCertMount + "/" + corev1.TLSPrivateKeyKey
-
-	consoleConfigMapSharedName = "console-config"
-	consoleConfigMapLocalName  = systemConfigPrefix + consoleConfigMapSharedName
-	consoleConfigKey           = consoleConfigMapSharedName + ".yaml"
-
-	ocpBrandingSecretName   = systemConfigPrefix + "ocp-branding-template"
-	ocpBrandingSecretMount  = systemConfigPathSecrets + "/" + ocpBrandingSecretName
-	ocpBrandingLoginPath    = ocpBrandingSecretMount + "/" + configv1.LoginTemplateKey
-	ocpBrandingProviderPath = ocpBrandingSecretMount + "/" + configv1.ProviderSelectionTemplateKey
-	ocpBrandingErrorPath    = ocpBrandingSecretMount + "/" + configv1.ErrorsTemplateKey
-
-	cliConfigNameAndKey = systemConfigPrefix + "cliconfig"
-	cliConfigMount      = systemConfigPathConfigMaps + "/" + cliConfigNameAndKey
-	cliConfigPath       = cliConfigMount + "/" + cliConfigNameAndKey
-
-	oauthMetadataName        = systemConfigPrefix + "metadata"
-	oauthMetadataAPIEndpoint = "/.well-known/oauth-authorization-server"
-
-	oauthBrowserClientName     = "openshift-browser-client"
-	oauthChallengingClientName = "openshift-challenging-client"
-
-	routerCertsSharedName = "router-certs"
-	routerCertsLocalName  = systemConfigPrefix + routerCertsSharedName
-	routerCertsLocalMount = systemConfigPathSecrets + "/" + routerCertsLocalName
-
-	servicePort   = 443
-	containerPort = 6443
+	deploymentVersionHashKey = "operator.openshift.io/rvs-hash"
 )
 
 // static environment variables from operator deployment
 var (
-	oauthserverImage   = os.Getenv(operandImageEnvName)
-	oauthserverVersion = os.Getenv(operandVersionEnvName)
-
-	operatorVersion = os.Getenv(operatorVersionEnvName)
+	oauthserverImage   = os.Getenv("IMAGE")
+	oauthserverVersion = os.Getenv("OPERAND_IMAGE_VERSION")
+	operatorVersion    = os.Getenv("OPERATOR_IMAGE_VERSION")
 
 	kasServicePort int
 )
 
 func init() {
 	var err error
-	kasServicePort, err = strconv.Atoi(os.Getenv(kasServicePortEnvName))
+	kasServicePort, err = strconv.Atoi(os.Getenv("KUBERNETES_SERVICE_PORT_HTTPS"))
 	if err != nil {
-		panic(err)
+		klog.Infof("defaulting KAS service port to 443 due to parsing error: %v", err)
+		kasServicePort = 443
 	}
 }
 
@@ -171,6 +84,10 @@ type authOperator struct {
 	console        configv1client.ConsoleInterface
 	infrastructure configv1client.InfrastructureInterface
 	ingress        configv1client.IngressInterface
+	apiserver      configv1client.APIServerInterface
+	proxy          configv1client.ProxyInterface
+
+	systemCABundle []byte
 
 	resourceSyncer resourcesynccontroller.ResourceSyncer
 }
@@ -194,7 +111,7 @@ func NewAuthenticationOperator(
 		versionGetter: versionGetter,
 		recorder:      recorder,
 
-		route: routeClient.Routes(targetNamespace),
+		route: routeClient.Routes("openshift-authentication"),
 
 		oauthClientClient: oauthClientClient.OAuthClients(),
 
@@ -209,15 +126,23 @@ func NewAuthenticationOperator(
 		console:        configClient.ConfigV1().Consoles(),
 		infrastructure: configClient.ConfigV1().Infrastructures(),
 		ingress:        configClient.ConfigV1().Ingresses(),
+		apiserver:      configClient.ConfigV1().APIServers(),
+		proxy:          configClient.ConfigV1().Proxies(),
 
 		resourceSyncer: resourceSyncer,
 	}
 
+	systemCABytes, err := ioutil.ReadFile("/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem")
+	if err != nil {
+		klog.Warningf("Unable to read system CA from /etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem: %v", err)
+	}
+	c.systemCABundle = systemCABytes
+
 	coreInformers := kubeInformersNamespaced.Core().V1()
 	configV1Informers := configInformers.Config().V1()
 
-	targetNameFilter := operator.FilterByNames(targetName)
-	configNameFilter := operator.FilterByNames(globalConfigName)
+	targetNameFilter := operator.FilterByNames("oauth-openshift")
+	configNameFilter := operator.FilterByNames("cluster")
 	prefixFilter := getPrefixFilter()
 
 	return operator.New("AuthenticationOperator2", c,
@@ -234,11 +159,13 @@ func NewAuthenticationOperator(
 		operator.WithInformer(configV1Informers.Consoles(), configNameFilter, controller.WithNoSync()),
 		operator.WithInformer(configV1Informers.Infrastructures(), configNameFilter, controller.WithNoSync()),
 		operator.WithInformer(configV1Informers.Ingresses(), configNameFilter, controller.WithNoSync()),
+		operator.WithInformer(configV1Informers.APIServers(), configNameFilter, controller.WithNoSync()),
+		operator.WithInformer(configV1Informers.Proxies(), configNameFilter, controller.WithNoSync()),
 	)
 }
 
 func (c *authOperator) Key() (metav1.Object, error) {
-	return c.authOperatorConfigClient.Client.Authentications().Get(globalConfigName, metav1.GetOptions{})
+	return c.authOperatorConfigClient.Client.Authentications().Get("cluster", metav1.GetOptions{})
 }
 
 func (c *authOperator) Sync(obj metav1.Object) error {
@@ -353,7 +280,9 @@ func (c *authOperator) handleSync(operatorConfig *operatorv1.Authentication) err
 
 	infrastructureConfig := c.handleInfrastructureConfig()
 
-	expectedCLIconfig, syncData, err := c.handleOAuthConfig(operatorConfig, route, routerSecret, service, consoleConfig, infrastructureConfig)
+	apiServerConfig := c.handleAPIServerConfig()
+
+	expectedCLIconfig, syncData, err := c.handleOAuthConfig(operatorConfig, route, routerSecret, service, consoleConfig, infrastructureConfig, apiServerConfig)
 	if err != nil {
 		return fmt.Errorf("failed handling OAuth configuration: %v", err)
 	}
@@ -372,12 +301,19 @@ func (c *authOperator) handleSync(operatorConfig *operatorv1.Authentication) err
 	// BLOCK 4: deployment
 	// ==================================
 
-	operatorDeployment, err := c.deployments.Deployments(targetNamespaceOperator).Get(targetNameOperator, metav1.GetOptions{})
+	if err := c.ensureBootstrappedOAuthClients("https://" + route.Spec.Host); err != nil {
+		return err
+	}
+
+	proxyConfig := c.handleProxyConfig()
+	resourceVersions = append(resourceVersions, "proxy:"+proxyConfig.Name+":"+proxyConfig.ResourceVersion)
+
+	operatorDeployment, err := c.deployments.Deployments("openshift-authentication-operator").Get("authentication-operator", metav1.GetOptions{})
 	if err != nil {
 		return err
 	}
 	// prefix the RV to make it clear where it came from since each resource can be from different etcd
-	resourceVersions = append(resourceVersions, "deployments:"+operatorDeployment.ResourceVersion)
+	resourceVersions = append(resourceVersions, "deployments:"+operatorDeployment.Name+":"+operatorDeployment.ResourceVersion)
 
 	configResourceVersions, err := c.handleConfigResourceVersions()
 	if err != nil {
@@ -390,6 +326,7 @@ func (c *authOperator) handleSync(operatorConfig *operatorv1.Authentication) err
 		operatorConfig,
 		syncData,
 		routerSecret,
+		proxyConfig,
 		operatorDeployment,
 		resourceVersions...,
 	)
@@ -472,8 +409,8 @@ func (c *authOperator) handleVersion(
 	// we have achieved our desired level
 	setProgressingFalse(operatorConfig)
 	setAvailableTrue(operatorConfig, "AsExpected")
-	c.setVersion(operatorSelfName, operatorVersion)
-	c.setVersion(targetName, oauthserverVersion)
+	c.setVersion("operator", operatorVersion)
+	c.setVersion("oauth-openshift", oauthserverVersion)
 
 	return nil
 }
@@ -508,7 +445,12 @@ func (c *authOperator) checkDeploymentReady(deployment *appsv1.Deployment, opera
 func (c *authOperator) checkRouteHealthy(route *routev1.Route, routerSecret *corev1.Secret, ingress *configv1.Ingress) (ready bool, msg, reason string, err error) {
 	caData := routerSecretToCA(route, routerSecret, ingress)
 
-	rt, err := transportFor("", caData, nil, nil)
+	// if systemCABundle is not empty, append the new line to the caData
+	if len(c.systemCABundle) > 0 {
+		caData = append(bytes.TrimSpace(caData), []byte("\n")...)
+	}
+
+	rt, err := transportFor("", append(caData, c.systemCABundle...), nil, nil)
 	if err != nil {
 		return false, "", "FailedTransport", fmt.Errorf("failed to build transport for route: %v", err)
 	}
@@ -538,13 +480,13 @@ func (c *authOperator) checkWellknownEndpointsReady(authConfig *configv1.Authent
 		return true, "", nil
 	}
 
-	caData, err := ioutil.ReadFile(rootCAFile)
+	caData, err := ioutil.ReadFile("/var/run/secrets/kubernetes.io/serviceaccount/ca.crt")
 	if err != nil {
 		return false, "", fmt.Errorf("failed to read SA ca.crt: %v", err)
 	}
 
 	// pass the KAS service name for SNI
-	rt, err := transportFor(kasServiceFullName, caData, nil, nil)
+	rt, err := transportFor("kubernetes.default.svc", caData, nil, nil)
 	if err != nil {
 		return false, "", fmt.Errorf("failed to build transport for SA ca.crt: %v", err)
 	}
@@ -565,7 +507,7 @@ func (c *authOperator) checkWellknownEndpointsReady(authConfig *configv1.Authent
 }
 
 func (c *authOperator) getAPIServerIPs() ([]string, error) {
-	kasService, err := c.services.Services(corev1.NamespaceDefault).Get(kasServiceAndEndpointName, metav1.GetOptions{})
+	kasService, err := c.services.Services(corev1.NamespaceDefault).Get("kubernetes", metav1.GetOptions{})
 	if err != nil {
 		return nil, fmt.Errorf("failed to get kube api server service: %v", err)
 	}
@@ -575,7 +517,7 @@ func (c *authOperator) getAPIServerIPs() ([]string, error) {
 		return nil, fmt.Errorf("unable to find kube api server service target port: %#v", kasService)
 	}
 
-	kasEndpoint, err := c.endpoints.Endpoints(corev1.NamespaceDefault).Get(kasServiceAndEndpointName, metav1.GetOptions{})
+	kasEndpoint, err := c.endpoints.Endpoints(corev1.NamespaceDefault).Get("kubernetes", metav1.GetOptions{})
 	if err != nil {
 		return nil, fmt.Errorf("failed to get kube api server endpoints: %v", err)
 	}
@@ -618,7 +560,7 @@ func subsetHasKASTargetPort(subset corev1.EndpointSubset, targetPort int) bool {
 }
 
 func (c *authOperator) checkWellknownEndpointReady(apiIP string, rt http.RoundTripper, route *routev1.Route) (bool, string, error) {
-	wellKnown := "https://" + apiIP + oauthMetadataAPIEndpoint
+	wellKnown := "https://" + apiIP + "/.well-known/oauth-authorization-server"
 
 	req, err := http.NewRequest(http.MethodGet, wellKnown, nil)
 	if err != nil {
@@ -653,7 +595,7 @@ func (c *authOperator) checkWellknownEndpointReady(apiIP string, rt http.RoundTr
 }
 
 func (c *authOperator) oauthClientsReady(route *routev1.Route) (bool, string, error) {
-	_, err := c.oauthClientClient.Get(oauthBrowserClientName, metav1.GetOptions{})
+	_, err := c.oauthClientClient.Get("openshift-browser-client", metav1.GetOptions{})
 	if err != nil {
 		if errors.IsNotFound(err) {
 			return false, "browser oauthclient does not exist", nil
@@ -661,7 +603,7 @@ func (c *authOperator) oauthClientsReady(route *routev1.Route) (bool, string, er
 		return false, "", err
 	}
 
-	_, err = c.oauthClientClient.Get(oauthChallengingClientName, metav1.GetOptions{})
+	_, err = c.oauthClientClient.Get("openshift-challenging-client", metav1.GetOptions{})
 	if err != nil {
 		if errors.IsNotFound(err) {
 			return false, "challenging oauthclient does not exist", nil
@@ -680,14 +622,14 @@ func (c *authOperator) setVersion(operandName, version string) {
 
 func defaultLabels() map[string]string {
 	return map[string]string{
-		"app": targetName,
+		"app": "oauth-openshift",
 	}
 }
 
 func defaultMeta() metav1.ObjectMeta {
 	return metav1.ObjectMeta{
-		Name:            targetName,
-		Namespace:       targetNamespace,
+		Name:            "oauth-openshift",
+		Namespace:       "openshift-authentication",
 		Labels:          defaultLabels(),
 		Annotations:     map[string]string{},
 		OwnerReferences: nil, // TODO
@@ -696,7 +638,7 @@ func defaultMeta() metav1.ObjectMeta {
 
 func defaultGlobalConfigMeta() metav1.ObjectMeta {
 	return metav1.ObjectMeta{
-		Name:   globalConfigName,
+		Name:   "cluster",
 		Labels: map[string]string{},
 		Annotations: map[string]string{
 			"release.openshift.io/create-only": "true",
@@ -705,9 +647,9 @@ func defaultGlobalConfigMeta() metav1.ObjectMeta {
 }
 
 func getPrefixFilter() controller.Filter {
-	names := operator.FilterByNames(targetName)
+	names := operator.FilterByNames("oauth-openshift")
 	prefix := func(obj metav1.Object) bool { // TODO add helper to combine filters
-		return names.Add(obj) || strings.HasPrefix(obj.GetName(), configVersionPrefix)
+		return names.Add(obj) || strings.HasPrefix(obj.GetName(), "v4-0-config-")
 	}
 	return controller.FilterFuncs{
 		AddFunc: prefix,
